/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2001 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gtkextra/gtkextra.h>
#include <scigraphica/sg.h>
#include "pixmaps/sg_small.xpm"

static gboolean ps_export_default			(SGpluginFile *plugin, 
                       		  	 		 const gchar *filename, 
                       			 		 FILE *opened,
                       			 		 GObject **object, 
					 		 gpointer data,
							 gboolean eps);

void
SGplot_ps_export_init(SGplugin *_plugin)
{
  GdkPixmap *pixmap;
  GdkBitmap *mask;
  SGpluginFile *plugin = SG_PLUGIN_FILE(_plugin);

  plugin->mode = SG_PLUGIN_FILE_EXPORT;
  plugin->nfilters = 1;
  sprintf(plugin->filter[0],".ps");

  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, gdk_colormap_get_system(), &mask, NULL, sg_small_xpm);
  plugin->pixmap = GTK_PIXMAP(gtk_pixmap_new(pixmap, mask));
  gdk_pixmap_unref(pixmap);
  gdk_bitmap_unref(mask);
}

void
SGplot_eps_export_init(SGpluginFile *plugin)
{
  GdkPixmap *pixmap;
  GdkBitmap *mask;

  plugin->mode = SG_PLUGIN_FILE_EXPORT;
  plugin->nfilters = 1;
  sprintf(plugin->filter[0],".eps");

  pixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, gdk_colormap_get_system(), &mask, NULL, sg_small_xpm);
  plugin->pixmap = GTK_PIXMAP(gtk_pixmap_new(pixmap, mask));
  gdk_pixmap_unref(pixmap);
  gdk_bitmap_unref(mask);
}

gboolean
SGplot_ps_export	(SGpluginFile *plugin, 
               		 const gchar *filename, 
              		 FILE *opened,
              		 GObject **object, gpointer data)
{
  return ps_export_default(plugin, filename, opened, object, data, FALSE);
}

gboolean
SGplot_eps_export	(SGpluginFile *plugin, 
               		 const gchar *filename, 
              		 FILE *opened,
              		 GObject **object, gpointer data)
{
  return ps_export_default(plugin, filename, opened, object, data, TRUE);
}

static gboolean
ps_export_default	(SGpluginFile *plugin, 
               		 const gchar *filename, 
              		 FILE *opened,
              		 GObject **object, gpointer data, gboolean eps)
{
  GList *list;
  SGplot *plot = SG_PLOT(*object);

  list = plot->layers;
  while(list){
    sg_layer_show_markers(SG_LAYER(list->data), FALSE);
    list = list->next;
  }

  gtk_plot_canvas_export_ps_with_size
                           (GTK_PLOT_CANVAS(plot),
                            filename,
                            plot->orientation,
                            eps,
                            GTK_PLOT_PSPOINTS,
                            plot->page_width,
                            plot->page_height);

  list = plot->layers;
  while(list){
    sg_layer_show_markers(SG_LAYER(list->data), TRUE);
    list = list->next;
  }

}

